from datetime import datetime, timedelta

from common.utils import logging
from common.utils.baseline.core import BaselineService
from common.utils.checkpoint.core import CheckpointService
from common.utils.event.core import EventService
from common.utils.flow.core import FlowService
from common.utils.lookup.core import LookupService
from common.utils.userconfig.core import UserConfigService
from trafficdrop.detector.core import TrafficDrop


def main():
    """
    Main function for detector

    Runs the detector for the current day, if it's 2am, run a second time
    for the previous day (give 2 hours for all the yesterday flow to arrive and rerun)
    """
    now = datetime.now()
    tdrop = init_trafficdrop_with_time(now)

    logger.info("starting traffic drop scan")
    tdrop.run()
    logger.info("ending traffic scan")

    catchup_run(now)


def init_trafficdrop_with_time(time: datetime) -> TrafficDrop:
    flow_service = FlowService()
    return TrafficDrop(
        BaselineService(),
        CheckpointService(),
        UserConfigService(),
        EventService(),
        flow_service,
        LookupService(flow_service),
        time,
    )


def catchup_run(now: datetime):
    if now.hour == 2:
        logger.info("running during hour 2, triggering run for yesterday")
        end_of_yesterday = datetime.combine(now.date() - timedelta(days=1), datetime.max.time())
        tdrop = init_trafficdrop_with_time(end_of_yesterday)

        logger.info("starting traffic drop for yesterday")
        tdrop.run()
        logger.info("ending traffic drop for yesterday")


if __name__ == "__main__":
    logger = logging.get_common_logger(__file__)
    logger.info("initializing traffic drop detector")
    main()
